/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.job;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.annotation.Resource;

import net.sourceforge.kivu4j.utils.lang.ConvertUtil;
import net.sourceforge.kivu4j.utils.lang.domain.NameValue;

import org.quartz.JobDetail;
import org.quartz.SchedulerException;
import org.quartz.SchedulerMetaData;
import org.quartz.Trigger;
import org.quartz.impl.StdScheduler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;



/**
 * 管理器控制实现
 *
 * @author Lucky Yang
 */
@Service("scheduleManager")
public class ScheduleManagerImpl implements ScheduleManager {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8837860595299204143L;
	protected Logger logger = LoggerFactory.getLogger(getClass());
	@Resource
	private StdScheduler scheduler;
	
	@Override
	public String[] getJobNames(String jobGroup) {
		Assert.notNull(jobGroup, "'jobGroup' must not be null");
		try {
			return this.scheduler.getJobNames(jobGroup);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public Trigger[] getTriggersOfJob(String jobName, String jobGroup) {
		Assert.notNull(jobName, "'jobName' must not be null");
		Assert.notNull(jobGroup, "'jobGroup' must not be null");
		try {
			return this.scheduler.getTriggersOfJob(jobName, jobGroup);
		} catch (SchedulerException e){
			throw new JobException(e);
		}
	}
	
	@Override
	public JobDetail createJob(String name, String group ,String description, Class<?> clazz) {
		Assert.notNull(name, "'name' must not be null");
		Assert.notNull(group, "'group' must not be null");		
		Assert.notNull(clazz, "'clazz' must not be null");
		
		try {
			JobDetail job = new JobDetail(name, group, clazz);
			job.setDescription(description);
			job.setDurability(true);
			return job;
		} catch (Exception e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public void unscheduleJob(String triggerName, String triggerGroup) {
		Assert.notNull(triggerName, "'triggerName' must not be null");
		Assert.notNull(triggerGroup, "'triggerGroup' must not be null");
		
		logger.info("unschedule trigger  " + triggerGroup + "." + triggerName);
		try {
			this.scheduler.unscheduleJob(triggerName, triggerGroup);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public void rescheduleJob(Trigger oldTrg, Trigger newTrg) {
		Assert.notNull(oldTrg, "'oldTrg' must not be null");	
		Assert.notNull(newTrg, "'newTrigger' must not be null");
		
		try {
				logger.info("reschedule trigger " +  newTrg.getFullName());
				
				this.scheduler.rescheduleJob(oldTrg.getName(), oldTrg.getGroup(), newTrg);			
		} catch (SchedulerException e) {
			throw new JobException(e);
		}		
	}


	@Override
	public JobDetail getJobDetail(String jobName, String jobGroup) {
		Assert.notNull(jobName, "'jobName' must not be null");
		Assert.notNull(jobGroup, "'jobGroup' must not be null");
		
		try {
			return this.scheduler.getJobDetail(jobName, jobGroup);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}


	@Override
	public Trigger getTrigger(String triggerName, String triggerGroup) {
		Assert.notNull(triggerName, "'triggerName' must not be null");
		Assert.notNull(triggerGroup, "'triggerGroup' must not be null");
		
		try {
			return this.scheduler.getTrigger(triggerName, triggerGroup);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}		
	}


	@Override
	public void scheduleJob(Trigger trigger) {
		Assert.notNull(trigger, "'trigger' must not be null");
		
		logger.info("schedule trigger " + trigger.getFullName());
		try {
			this.scheduler.scheduleJob(trigger);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}	
		
	}

	@Override
	public Collection<NameValue> getMetaData() {
		List<NameValue> rslt = new ArrayList<NameValue>();
		SchedulerMetaData md = this.scheduler.getMetaData();
		
		rslt.add(new NameValue("started", md.isStarted()));
		rslt.add(new NameValue("shutdown", md.isShutdown()));
		rslt.add(new NameValue("schedName", md.getSchedulerName()));
		rslt.add(new NameValue("schedInst", md.getSchedulerInstanceId()));
		rslt.add(new NameValue("schedClass", md.getSchedulerClass().getName()));
		rslt.add(new NameValue("isRemote", md.isSchedulerRemote()));
		rslt.add(new NameValue("isInStandbyMode", md.isInStandbyMode()));
		rslt.add(new NameValue("startTime", ConvertUtil.convert(md.getRunningSince(),String.class)));
		rslt.add(new NameValue("numJobsExec", md.getNumberOfJobsExecuted()));
		rslt.add(new NameValue("jsClass", md.getJobStoreClass().getName()));
		rslt.add(new NameValue("jsPersistent", md.isJobStoreSupportsPersistence()));
		rslt.add(new NameValue("jsClustered", md.isJobStoreClustered()));
		rslt.add(new NameValue("tpClass", md.getThreadPoolClass().getName()));
		rslt.add(new NameValue("tpSize", md.getThreadPoolSize()));
		rslt.add(new NameValue("version", md.getVersion()));
				
		return rslt;
	}

	@Override
	public void resumeAll() {
		logger.info("resume all jobs");
		try {
			this.scheduler.resumeAll();
		} catch (SchedulerException e) {
			throw new JobException(e);
		}		
	}

	@Override
	public void pauseAll() {
		logger.info("pause all jobs");
		try {
			this.scheduler.pauseAll();
		} catch (SchedulerException e) {
			throw new JobException(e);
		}	
	}

	@Override
	public void shutdown() {
		logger.info("shutdown scheduler");
		
		try {
			this.scheduler.shutdown(false);
		} catch (Exception e) {
			throw new JobException(e);
		}	
	}

	@Override
	public void startup() {
		logger.info("start scheduler");
		
		try {
			this.scheduler.start();
		} catch (SchedulerException e) {
			throw new JobException(e);
		}	
	}

	@Override
	public void standby() {
		logger.info("standby scheduler");
		
		try {
			this.scheduler.standby();
		} catch (Exception e) {
			throw new JobException(e);
		}		
	}

	@Override
	public void active() {	
		logger.info("start scheduler");
		try {
			this.scheduler.start();
		} catch (Exception e) {
			throw new JobException(e);
		}			
	}

	@Override
	public int getTriggerState(String name, String group) {
		Assert.notNull(name, "'name' must not be null");
		Assert.notNull(group, "'group' must not be null");
		try {
			return this.scheduler.getTriggerState(name, group);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}

	@Override
	public void resumeTrigger(String name, String group) {
		Assert.notNull(name, "'name' must not be null");
		Assert.notNull(group, "'group' must not be null");
		
		logger.info("resume trigger " + name + "." + group);
		try {
			this.scheduler.resumeTrigger(name, group);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public void pauseTrigger(String name, String group) {
		Assert.notNull(name, "'name' must not be null");
		Assert.notNull(group, "'group' must not be null");
		
		logger.info("pause trigger " + name + "." + group);
		try {
			this.scheduler.pauseTrigger(name, group);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public void resumeGroup(String group) {
		Assert.notNull(group, "'group' must not be null");
		
		logger.info("resume group " + group);
		try {
			this.scheduler.resumeJobGroup(group);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}
	
	@Override
	public void pauseGroup(String group) {
		Assert.notNull(group, "'group' must not be null");
		
		logger.info("pause group " + group);
		try {
			this.scheduler.pauseJobGroup(group);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}
	}

	@Override
	public void addJob(JobDetail job) {
		Assert.notNull(job, "'jobName' must not be null");

		logger.info("add job " + job.getFullName());
		try {
			this.scheduler.addJob(job, false);
		} catch (SchedulerException e) {
			throw new JobException(e);
		}		
	}
	
	@Override
	public void addJob(String name, String group, Class<?> clazz,
			String description) {
		JobDetail job = this.getJobDetail(name, group);
		// 判断是否不存在
		if (job == null){
			job = this.createJob(name, group, description, clazz);
			this.addJob(job);
		}	
	}

	@Override
	public boolean isStandby() {
		return this.scheduler.isInStandbyMode();
	}
}
